package com.ruoyi.app.service.impl;

import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.app.domain.TbFamilyMember;
import com.ruoyi.app.domain.vo.AppMemberVo;
import com.ruoyi.app.mapper.TbAppUserMapper;
import com.ruoyi.app.service.ITbAppUserService;
import com.ruoyi.app.service.ITbFamilyMemberService;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.bean.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * 会员Service业务层处理
 *
 * @author Alex
 * @date 2020-09-24
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TbAppUserServiceImpl extends ServiceImpl<TbAppUserMapper, TbAppUser> implements ITbAppUserService {

    @Autowired
    private ITbFamilyMemberService memberService;
    @Autowired
    private ITbAppUserService userService;

    /**
     * 获取所有未删除状态的会员信息列表
     * @return
     */
    @Override
    public List<AppMemberVo> selectAppMembers(String enable, String search, String beginTime, String endTime){
        return baseMapper.selectAppMembers("user",enable, search, beginTime, endTime);
    }

    /**
     * 添加/修改会员
     * @param memberVo
     * @return
     */
    @Override
    public boolean saveOrUpdate(AppMemberVo memberVo){
        if (memberVo == null) {
            return false;
        }
        TbAppUser appUser = new TbAppUser();
        TbFamilyMember member = new TbFamilyMember();
        Date date = new Date();

        BeanUtils.copyBeanProp(appUser,memberVo);
        BeanUtils.copyBeanProp(member,memberVo);
        appUser.setUpdateBy(memberVo.getUserUpdateBy());
        appUser.setUpdateTime(date);
        appUser.setRemark(memberVo.getUserRemark());
        member.setUpdateBy(memberVo.getUserUpdateBy());
        member.setUpdateTime(date);
        //新增用户
        if (memberVo.getUserId() == null) {
            appUser.setQrcode(DateUtils.getCurrentTimeRandom());
            appUser.setCreateBy(memberVo.getUserCreateBy());
            appUser.setCreateTime(date);
            userService.save(appUser);
        }else {// 否则修改用户
            appUser.setId(memberVo.getUserId());
            userService.updateById(appUser);
        }
        // 新增成员
        if (memberVo.getMemberId() == null) {
            member.setAppUserId(appUser.getId());
            member.setCreateBy(memberVo.getUserCreateBy());
            member.setCreateTime(date);
            memberService.save(member);
        } else { // 否则修改成员
            member.setId(memberVo.getMemberId());
            memberService.updateById(member);
        }
        return true;
    }

    @Override
    public AppMemberVo getAppMember(String id) {
        return baseMapper.getAppMember("user", id);
    }
}
